.TH FCOMP 1
.SH FCOMP
fcomp \- file comparison utility
.SH SYNOPSIS
.B fcomp
file1 file2 [options]
.SH DESCRIPTION
The
.I fcomp
program is a file comparison utility which works similarly to the diff utility.
Its advantage is that it always produces minimal differences,
and so will never mis-sync when comparing files.
Its disadvantage is that it runs slower due to the extra work
required to produce optimal differences.
However, for files differing by less than a few thousand lines,
its performance is adequate.
The algorithms used by this utility are also used by the
.I fsc
utility in order to produce the edit history.
.PP
To compare file "old" to file "new", the command:
.sp
.nf
    fcomp old new
.fi
.sp
would be used.
This gives the differences involved in converting
FROM file
.I old
TO file
.IR new .
This is analogous to the use of the
.I cp
command.
Either the old or new file may be a directory,
in which case the comparison is done to the file in the directory
with the same name as the other file.
An error is given if both old and new are directories.
.PP
The normal output from
.I fcomp
starts with three lines which indicate the two file names and a totals line.
The file names are labeled 'A' and 'B'.
The totals line indicates the number of lines which need to be inserted and
deleted in order to change the first file into the second file.
It also lists the number of matching lines.
.PP
Following the totals line is the list of changed lines, which specify
what is deleted, inserted, or changed, along with the line numbers in
the 'A' and 'B' files where the change is made.
.PP
There are options which change this default output format to either show
both files with "change bars", or to create a machine-readable script
describing the changes.
.SH OPTIONS
.TP
.I \-b
This ignores blank lines for the comparison.
Their line numbers are still important, but they do not contribute as
differences between the files.
.TP
.I \-c
This specifies the number of lines of "context" which is displayed.
This shows the specified number of lines before and after
the actual lines being changed.
This is useful to locate and identify the line which is actually being changed,
when there are many identical copies of the line in the file.
.TP
.I \-e
This outputs a machine-readable output of the differences between the
two files.
The output does not contain any of the text from the two files.
Instead, it contains editing instructions on how to convert the first
file's lines into the second file's lines.
Each line begins with an identifying letter, followed by additional data.
The first two lines start with 'A' and 'B', and specify the two file names
being compared.
This is followed by lines starting with the letters 'I', 'D', or 'R',
describing insertions, deletions,
and replacements in going from the first to the second file.
Each of these letters is followed by two blank separated line numbers
(or line number ranges separated by a dash).
The first line number is the location in the first file of the change,
and the second line number is the location in the second file of the change.
The last line in the file starts with 'T', and is a totals line.
It specifies the number of inserted lines, deleted lines, and unchanged lines.
(Replaced lines count both as insertions and deletions.)
.TP
.I \-f
This stops the comparison if the number of changes exceeds
the specified number.
Each change is a delete or insert of a single line.
This is useful when you are not interested in the results
when the files are totally different.
Another use is a quick check to see if two files are identical,
by using a value of zero.
.TP
.I \-h
This displays a usage of all the options.
.TP
.I \-j
This merges together lines which have changed,
if they are separated by up to the specified number of unchanged lines.
This makes a change look bigger, but reduces the "choppiness"
of the output by showing fewer regions being changed.
This is particularly effective to suppress worthless matchings
of single blank lines or comment beginning and ending lines.
A useful value for this option is 3 or so.
.TP
.I \-m
This shows lines which are the same between the two files,
instead of the lines which are different.
.TP
.I \-n
This outputs the line numbers at the left edge of the output.
This isn't normally needed,
since the line numbers are displayed in the comment line
preceding the lines being displayed.
Not outputting the line numbers prevents the terminal from
needlessly scrolling for long lines.
.TP
.I \-o
This outputs the results to the specified file name instead of stdout.
.TP
.I \-q
This outputs only a one-line summary of the differences between the files.
This gives the number of lines that need to be inserted and deleted
in going from the first to the second file,
and the number of lines which do not need changing.
Lines which are changed count as both an insert and a delete.
.TP
.I \-s
This ignores differences in the number of spaces in the two lines.
That is, two or more adjacent spaces are handled as a single space.
Spaces at the beginning or end of a line are totally ignored.
.TP
.I \-u
This uppercases lines from both files before comparing.
This removes any lines which differ only in case from the output.
.TP
.I \-v
This causes
.I fcomp
to print verbose output on what it is doing during the comparison.
.TP
.I \-w
This outputs all of both files together,
showing what happened to each line of the first file in order to change
to the line in the second file.
This output is in "change bar" format, where inserted lines
begin with "|+\ \ ",
deleted lines begin with "|-\ \ ",
and unchanged lines begin with "\ \ \ \ ".
The presence of the vertical bar in the first column
makes it easy to search for the changed lines.
.SH AUTHOR
.nf
David I. Bell
dbell@canb.auug.org.au
