/*
 * Copyright (c) 2000 David I. Bell
 * Permission is granted to use, distribute, or modify this source,
 * provided that this copyright notice remains intact.
 *
 * Definitions for file handling routines.
 */

#ifndef	FIO_H
#define	FIO_H


#include "bool.h"


#define FIO_READ	0	/* open old for reading */
#define FIO_WRITE	1	/* open new file for writing (random access) */
#define FIO_UPDATE	2	/* open old file for read and write */
#define FIO_SEQWRITE	3	/* open new file for writing (sequential) */


typedef struct
{
	int	unit;		/* file unit */
	int	valid;		/* amount of valid data */
	int	offset;		/* current offset within the block */
	int	dirty;		/* offset within block needing writing */
	int	errorCode;	/* latest error code for errors */
	BOOL	eofFlag;	/* TRUE if end of file detected */
	BOOL	canRead;	/* TRUE if reads allowed */
	long	pos;		/* byte position of beginning of block */
	char *	data;		/* current file data block */
} FIO;

#define	FIO_NULL	((FIO *) 0)


/*
 * Macros to return information
 */
#define	FioPos(fp)	((fp)->pos + (fp)->offset)
#define	FioError(fp)	((fp)->errorCode)
#define	FioEof(fp)	((fp)->eofFlag)


/*
 * Procedures
 */
extern	BOOL	FioOpen(FIO *, const char *, int);
extern	BOOL	FioClose(FIO *);
extern	BOOL	FioSeek(FIO *, long);
extern	BOOL	FioSkip(FIO *, long);
extern	char *	FioReadline(FIO *, long *, BOOL);
extern	long	FioRead(FIO *, char *, long);
extern	BOOL	FioFormat(FIO *, const char *, ...);
extern	BOOL	FioStr(FIO *, const char *);
extern	BOOL	FioChar(FIO *, int);
extern	BOOL	FioWrite(FIO *, const char *, long);
extern	BOOL	FioCopy(FIO *, FIO *, long);
extern	BOOL	FioType(FIO *);
extern	BOOL	FioFlush(FIO *);
extern	void	FioReset(void);

#endif

/* END CODE */
